@@ -4,9 +4,11 @@ cv_GIC_indexed <- function(X, y, nfolds, model_function, ...) {
 
         if (family == "gaussian"){
                 n <- length(y)
-                real_n <- 0 #recount  of test instances
+                total_n <- 0 #recount  of test instances
                 foldid <- sample(rep(1:nfolds,length.out=n))   #PP replaces cvfolds by a simpler sample(rep()) function
-                err <- list(); rss <- list(); #md <- list()
+                err <- list()
+                rss <- list() #md <- list()
+                fold_n <-list()
 
                 model.full <- model_function(X, y, ...)
                 lambda.full<- model.full$lambda
@@ -18,13 +20,14 @@ cv_GIC_indexed <- function(X, y, nfolds, model_function, ...) {
                         Xtr <- X[foldid != fold, ,drop = FALSE]
                         ytr <- y[foldid != fold, drop = FALSE]
 
-                        compute_model <- cv_compute_model(model_function, Xtr, ytr, Xte, yte, real_n, lambda.full = lambda.full, ...)   #three letter abbreviations (lambda.full vs lam) make this function call confused, so explicit passing of named parameter i.e. lambda.full=lambda.full is required
+                        compute_model <- cv_compute_model(model_function, Xtr, ytr, Xte, yte, lambda.full = lambda.full, ...)   #three letter abbreviations (lambda.full vs lam) make this function call confused, so explicit passing of named parameter i.e. lambda.full=lambda.full is required
                         model<-compute_model$model
                         Xtr<-compute_model$Xtr
                         ytr<-compute_model$ytr
                         Xte<-compute_model$Xte
                         yte<-compute_model$yte
-                        real_n<-compute_model$real_n
+                        total_n <- total_n + compute_model$this_fold_n
+                        fold_n[[fold]] <- compute_model$this_fold_n
 
                         #PP new code
                         rss[[fold]] <- model$rss
@@ -38,11 +41,12 @@ cv_GIC_indexed <- function(X, y, nfolds, model_function, ...) {
 
                 len_err <- sapply(err, length)
                 foldmin <- min(len_err)
-                ERR <- sapply(1:nfolds, function(i) err[[i]][ (len_err[i] - foldmin + 1) : len_err[i] ] )
+                ERR <- sapply(1:nfolds, function(i) utils::tail(err[[i]], foldmin))
                 if (foldmin == 1) {
                   ERR<-t(as.matrix((ERR)))  #making it a horizontal one-row matrix
                 }
                 #err <- rowMeans(ERR); kt <- which(err == min(err)); df.min <- dmr$df[kt[length(kt)]]; plot(err, type="o")
+                # ERR[m, f] = CV error for model `m` in fold `f`
 
                 p1 <- model.full$df[1]
                 s2 <- model.full$rss[1]/(n-p1)
@@ -54,20 +58,32 @@ cv_GIC_indexed <- function(X, y, nfolds, model_function, ...) {
                 RIC_constant <- constants()$RIC_gaussian_constant
                 Const <- exp(seq(log(RIC_constant/50),log(RIC_constant*50), length=81))
                 laGIC <- Const*log(p)*s2
-                RSS <- sapply(1:nfolds, function(i) rss[[i]][ (len_err[i] - foldmin + 1) : len_err[i] ] )
+                RSS <- sapply(1:nfolds, function(i) utils::tail(rss[[i]], foldmin))
                 if (is.null(dim(RSS))) {
                   RSS<-t(as.matrix((RSS)))  #making it a horizontal one-row matrix
                 }
+                # RSS[m, f] = CV rss for model `m` in fold `f`
+
+                error_means_in_folds <- sweep(RSS, 2, unlist(fold_n)^2*s2, "/")  # For each column (i.e., fold) of the matrix, divide it by the corresponding entry in a vector
+                # error_means_in_folds is a matrix with foldmin rows and nfolds column,
+                # containing mean errors (in term of rss) within each model size and in each fold
+
+                error_means <- rowSums(RSS)/(total_n^2*s2)    # a sum of errors (in term of rss) across all folds divided by the total number of test cases
+                # error_means is a vector sized foldmin now and it stores mean classification errors (in term of rss) for different model sizes
+                # note that it is correctly weighted already
+
                 #MD <- sapply(1:nfolds, function(i)  md[[i]][ (len_err[i] - foldmin + 1) : len_err[i] ] )
                 IND <- apply( RSS, 2, function(r) sapply( laGIC, function(la) which.min(r+la*length(r):1) ) )
-                errGIC <- apply( IND, 1, function(ind) mean(ERR[cbind(ind,1:nfolds)]) )
+                errGIC <- apply( IND, 1, function(ind) mean(ERR[cbind(ind,1:nfolds)]))
+                #errGIC - vector of mean CV error values for models selected by each GIC penalty level
+
                 #mdGIC  <- apply( IND, 1, function(ind) mean(MD[cbind(ind,1:10)]) )
                 #plot(mdGIC[length(laGIC):1],errGIC[length(laGIC):1]/s2, xlab="MD", ylab="PE", type="o")
 
                 r <- model.full$rss
                 kt <- which(errGIC == min(errGIC))
-                indGIC <- kt[length(kt)]    #TODO: why last?
-                gic.full <- (r+laGIC[indGIC]*length(r):1)/(real_n*s2)
+                indGIC <- kt[length(kt)]    # why last? GIC penalties grow → higher k implies simpler models
+                gic.full <- (r+laGIC[indGIC]*length(r):1)/(total_n*s2)
                 #plot(gic.full[length(gic.full):1])
 
         } else{
@@ -81,7 +97,7 @@ cv_GIC_indexed <- function(X, y, nfolds, model_function, ...) {
                         }
                         n1 <- table(y)[1]
                         n2 <- table(y)[2]
-                        real_n <- 0 #recount  of test instances
+                        total_n <- 0 #recount  of test instances
 
                         foldid1 <- sample(rep(1:nfolds,length.out=n1))  #PP replaces cvfolds by a simpler sample(rep()) function
                         foldid2 <- sample(rep(1:nfolds,length.out=n2))  #PP replaces cvfolds by a simpler sample(rep()) function
@@ -89,7 +105,10 @@ cv_GIC_indexed <- function(X, y, nfolds, model_function, ...) {
                         foldid[which(y == levels(factor(y))[1])] = foldid1
                         foldid[which(y == levels(factor(y))[2])] = foldid2
                         #PP new code error <- list()
-                        err <- list(); loglik <- list(); #md <- list()
+                        err <- list()
+                        loglik <- list()
+                        #md <- list()
+                        fold_n <-list()
 
                         model.full <- model_function(X, y, ...)
                         lambda.full<- model.full$lambda
@@ -101,32 +120,32 @@ cv_GIC_indexed <- function(X, y, nfolds, model_function, ...) {
                                 Xtr <- X[foldid != fold, , drop = FALSE]
                                 ytr <- y[foldid != fold, drop = FALSE]
 
-                                compute_model <- cv_compute_model(model_function, Xtr, ytr, Xte, yte, real_n, lambda.full = lambda.full, ...)   #three letter abbreviations (lambda.full vs lam) make this function call confused, so explicit passing of named parameter i.e. lambda.full=lambda.full is required
+                                compute_model <- cv_compute_model(model_function, Xtr, ytr, Xte, yte, lambda.full = lambda.full, ...)   #three letter abbreviations (lambda.full vs lam) make this function call confused, so explicit passing of named parameter i.e. lambda.full=lambda.full is required
                                 model<-compute_model$model
                                 Xtr<-compute_model$Xtr
                                 ytr<-compute_model$ytr
                                 Xte<-compute_model$Xte
                                 yte<-compute_model$yte
-                                real_n<-compute_model$real_n
+                                total_n <- total_n + compute_model$this_fold_n
+                                fold_n[[fold]] <- compute_model$this_fold_n
 
-                                #SzN new code based on PP new code
                                 loglik[[fold]] <- -2*model$loglik
 
                                 pred <- predict.DMR(model, newx = as.data.frame(Xte), type = "class")
-                                #SzN new code based on PP new code error[[fold]] <- apply(pred, 2, function(z) sum(z != yte))
+
                                 err[[fold]] <- apply(pred, 2, function(z) mean(z != yte))
 
                         }
 
                         len_err <- sapply(err, length)
                         foldmin <- min(len_err)
-                        ERR <- sapply(1:nfolds, function(i) err[[i]][ (len_err[i] - foldmin + 1) : len_err[i] ] )
-                        #err <- rowMeans(ERR); kt <- which(err == min(err)); df.min <- dmr$df[kt[length(kt)]]; plot(err, type="o")
-
-
+                        ERR <- sapply(1:nfolds, function(i) utils::tail(err[[i]], foldmin))
                         if (foldmin == 1) {
                           ERR<-t(as.matrix((ERR)))  #making it a horizontal one-row matrix
                         }
+                        # ERR[m, f] = CV error for model `m` in fold `f`
+
+                        #err <- rowMeans(ERR); kt <- which(err == min(err)); df.min <- dmr$df[kt[length(kt)]]; plot(err, type="o")
 
                         p <- ncol(model.full$beta)
                         if (is.null(p))
@@ -135,20 +154,33 @@ cv_GIC_indexed <- function(X, y, nfolds, model_function, ...) {
                         RIC_constant <- constants()$RIC_binomial_constant
                         Const <- exp(seq(log(RIC_constant/50),log(RIC_constant*50), length=81))
                         laGIC <- Const*log(p)
-                        LOGLIK <- sapply(1:nfolds, function(i) loglik[[i]][ (len_err[i] - foldmin + 1) : len_err[i] ] )
+                        LOGLIK <- sapply(1:nfolds, function(i) utils::tail(loglik[[i]], foldmin))
                         if (is.null(dim(LOGLIK))) {
                           LOGLIK<-t(as.matrix((LOGLIK)))  #making it a horizontal one-row matrix
                         }
+                        # LOGLIK[m, f] = CV loglik for model `m` in fold `f`
+
+                        error_means_in_folds <- sweep(-2*LOGLIK, 2, unlist(fold_n), "/")  # For each column (i.e., fold) of the matrix, divide it by the corresponding entry in a vector
+                        # error_means_in_folds is a matrix with foldmin rows and nfolds column,
+                        # containing mean errors (in term of logliks) within each model size and in each fold
+
+                        error_means <- -2*rowSums(LOGLIK)/total_n   # a sum of errors (in term of logliks) across all folds divided by the total number of test cases
+                        # error_means is a vector sized foldmin now and it stores mean classification errors (in term of logliks) for different model sizes
+                        # note that it is correctly weighted already
+
+
                         #MD <- sapply(1:nfolds, function(i)  md[[i]][ (len_err[i] - foldmin + 1) : len_err[i] ] )
                         IND <- apply( LOGLIK, 2, function(ll) sapply( laGIC, function(la) which.min(ll+la*length(ll):1) ) )
-                        errGIC <- apply( IND, 1, function(ind) mean(ERR[cbind(ind,1:nfolds)]) )
+                        errGIC <- apply( IND, 1, function(ind) mean(ERR[cbind(ind,1:nfolds)]))
+                        ##errGIC - vector of mean CV error values for models selected by each GIC penalty level
+
                         #mdGIC  <- apply( IND, 1, function(ind) mean(MD[cbind(ind,1:10)]) )
                         #plot(mdGIC[length(laGIC):1],errGIC[length(laGIC):1]/s2, xlab="MD", ylab="PE", type="o")
 
                         ll <- -2*model.full$loglik
                         kt <- which(errGIC == min(errGIC))
-                        indGIC <- kt[length(kt)]    #TODO: why last?
-                        gic.full <- (ll+laGIC[indGIC]*length(ll):1)/real_n
+                        indGIC <- kt[length(kt)]    # why last? GIC penalties grow → higher k implies simpler models
+                        gic.full <- (ll+laGIC[indGIC]*length(ll):1)/total_n
                         #plot(gic.full[length(gic.full):1])
 
                 }
@@ -156,12 +188,31 @@ cv_GIC_indexed <- function(X, y, nfolds, model_function, ...) {
                         stop("Error: wrong family, should be one of: gaussian, binomial")
                 }
         }
+
+        # first, compute .min estimator
         kt <- which(gic.full == min(stats::na.omit(gic.full))) #kt stores indexes in error equal to a minimum error.
         #if there is more than one such index, the LAST one is the one returned, because LAST means a smaller model.
         indMod <- kt[length(kt)]
         df.min <- model.full$df[indMod]
 
-        kt <- which(gic.full <= min(stats::na.omit(gic.full)) + stats::sd(stats::na.omit(gic.full[gic.full!=Inf & gic.full!=-Inf])))
+        # next, compute .1se estimator
+        wt <- unlist(fold_n) / total_n
+        std_err <- sqrt(apply(error_means_in_folds^2-error_means^2, 1, stats::weighted.mean, wt)) / sqrt(nfolds-1)
+        # std_err is a vector (of length foldmin) and it stores standard errors (in terms of logliks or rss) for different model sizes
+        # calculated similarly as in glmnet for group=TRUE, i.e. doubly weighted (once for mean calculation and then for sd calculation)
+
+        # BUT: foldmin may be shorter than the length of a model.full (and of gic.full, which is the same length)
+        # We need to propagate the first value to the left
+        if (length(gic.full) > length(std_err))
+          std_err <- c(rep(std_err[1], length(gic.full) - length(std_err)), std_err)
+
+        # ALSO: foldmin may be longer than the length of a model.full (and of gic.full, which is the same length)
+        # This is rare but may happen, we need to shorten std_err in this case
+        if (length(gic.full) < length(std_err))
+          std_err <- utils::tail(std_err, length(gic.full))
+
+
+        kt <- which(gic.full <= gic.full[kt] + std_err[kt])
         if (length(kt) == 0) {
           df.1se <- NULL
         } else {
@@ -169,6 +220,6 @@ cv_GIC_indexed <- function(X, y, nfolds, model_function, ...) {
           df.1se <- model.full$df[indMod]
         }
 
-        out <- list(df.min = df.min, df.1se = df.1se, dmr.fit = model.full, cvm = gic.full, foldid = foldid)
+        out <- list(df.min = df.min, df.1se = df.1se, dmr.fit = model.full, cvm = gic.full, cvse = std_err, foldid = foldid)
         return(out)
 }
